import * as vue from 'vue';
import { Ref } from 'vue';
import { SWR, SWRKey, SWROptions, SWRMutateOptions, SWRRevalidateOptions, CacheClearOptions } from 'swrev';

declare class VSWR extends SWR {
    /**
     * Use the SWR with a vue application.
     */
    useSWR<D = any, E = Error>(key: SWRKey | undefined | (() => SWRKey | undefined), options?: Partial<SWROptions<D>>): {
        data: Ref<D | undefined>;
        error: Ref<E | undefined>;
        mutate: (value: D, ops?: Partial<SWRMutateOptions<D>> | undefined) => void;
        revalidate: (ops?: Partial<SWRRevalidateOptions<D>> | undefined) => void;
        clear: (ops?: Partial<CacheClearOptions> | undefined) => void;
        stop: vue.WatchStopHandle;
        isLoading: vue.ComputedRef<boolean>;
        isValid: vue.ComputedRef<boolean>;
        loading: () => Promise<Ref<D>>;
    };
}
/**
 * Creates a mew SWR instance and exports basic methods to
 * work with without the need for method calling.
 */
declare const createSWR: <D = any>(options?: Partial<SWROptions<D>> | undefined) => VSWR;
/**
 * Default SWR instance. Can be replaced
 * with the `createDefaultSWR` function.
 */
declare let swr: VSWR;
/**
 * Creates and sets a default SWR instance given
 * the options.
 */
declare const createDefaultSWR: <D = any>(options?: Partial<SWROptions<D>> | undefined) => VSWR;
/**
 * Gets the data of the given key. Keep in mind
 * this data will be stale and revalidate in the background
 * unless specified otherwise.
 */
declare const subscribe: <D>(key: SWRKey | undefined, onData: (value: D) => any) => () => void;
/**
 * Subscribes to errors on the given key.
 */
declare const subscribeErrors: <E>(key: SWRKey | undefined, onError: (error: E) => any) => () => void;
/**
 * Gets the current cached data of the given key.
 * This does not trigger any revalidation nor mutation
 * of the data.
 * - If the data has never been validated
 * (there is no cache) it will return undefined.
 * - If the item is pending to resolve (there is a request
 * pending to resolve) it will return undefined.
 */
declare const get: <D = any>(key?: string | undefined) => D | undefined;
/**
 * Gets an element from the cache. The difference
 * with the get is that this method returns a promise
 * that will resolve the the value. If there's no item
 * in the cache, it will wait for it before resolving.
 */
declare const getOrWait: <D = any>(key: SWRKey) => Promise<D>;
/**
 * Use a SWR value given the key and
 * subscribe to future changes.
 */
declare const use: <D = any, E = Error>(key: SWRKey | undefined | (() => SWRKey | undefined), onData: (value: D) => void, onError: (error: E) => void, options?: Partial<SWROptions<D>> | undefined) => {
    unsubscribe: () => void;
};
/**
 * Use the SWR with a vue application.
 */
declare const useSWR: <D = any, E = Error>(key: SWRKey | (() => SWRKey), options?: Partial<SWROptions<D>> | undefined) => {
    data: Ref<D | undefined>;
    error: Ref<E | undefined>;
    mutate: (value: D, ops?: Partial<SWRMutateOptions<D>> | undefined) => void;
    revalidate: (ops?: Partial<SWRRevalidateOptions<D>> | undefined) => void;
    clear: (ops?: Partial<CacheClearOptions> | undefined) => void;
    stop: vue.WatchStopHandle;
    isLoading: vue.ComputedRef<boolean>;
    isValid: vue.ComputedRef<boolean>;
    loading: () => Promise<Ref<D>>;
};
/**
 * Mutates the data of a given key with a new value.
 * This is used to replace the cache contents of the
 * given key manually.
 */
declare const mutate: <D = any>(key: SWRKey, value: D, options?: Partial<SWRMutateOptions<D>> | undefined) => void;
/**
 * Revalidates the key and mutates the cache if needed.
 */
declare const revalidate: <D>(key: SWRKey, options?: Partial<SWRRevalidateOptions<D>> | undefined) => void;
/**
 * Clear the specified keys from the cache. If no keys
 * are specified, it clears all the cache keys.
 */
declare const clear: (keys?: string | string[] | undefined, options?: Partial<CacheClearOptions> | undefined) => void;

export { VSWR, clear, createDefaultSWR, createSWR, get, getOrWait, mutate, revalidate, subscribe, subscribeErrors, swr, use, useSWR };
